Package: eddington
Title: Compute a Cyclist's Eddington Number
Version: 4.3.0
Authors@R: c(
    person('Paul', 'Egeler', email = 'paulegeler@gmail.com', role = c('aut','cre')),
    person('Tashi', 'Reigle', role = 'ctb'))
Description: Compute a cyclist's Eddington number, including efficiently
    computing cumulative E over a vector. A cyclist's Eddington number
    <https://en.wikipedia.org/wiki/Arthur_Eddington#Eddington_number_for_cycling>
    is the maximum number satisfying the condition such that a cyclist has
    ridden E miles or greater on E distinct days. The algorithm in this package
    is an improvement over the conventional approach because both summary
    statistics and cumulative statistics can be computed in linear time, since
    it does not require initial sorting of the data. These functions may also be
    used for computing h-indices for authors, a metric described by Hirsch (2005)
    <doi:10.1073/pnas.0507655102>. Both are specific applications of computing
    the side length of a Durfee square <https://en.wikipedia.org/wiki/Durfee_square>.
    Some additional author-level metrics such as g-index and i10-index are also
    included in the package.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.2.0)
LinkingTo: Rcpp
Imports: Rcpp, R6, methods, xml2
Suggests: testthat, knitr, rmarkdown, stats, dplyr, tibble
SystemRequirements: C++17
VignetteBuilder: knitr
RoxygenNote: 7.3.2
URL: https://github.com/pegeler/eddington2
BugReports: https://github.com/pegeler/eddington2/issues
NeedsCompilation: yes
Packaged: 2026-04-12 03:16:10 UTC; pablo
Author: Paul Egeler [aut, cre],
  Tashi Reigle [ctb]
Maintainer: Paul Egeler <paulegeler@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 21:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-13 23:27:06 UTC; unix
Archs: eddington.so.dSYM

Package: eddington
Title: Compute a Cyclist's Eddington Number
Version: 4.3.0
Authors@R: c(
    person('Paul', 'Egeler', email = 'paulegeler@gmail.com', role = c('aut','cre')),
    person('Tashi', 'Reigle', role = 'ctb'))
Description: Compute a cyclist's Eddington number, including efficiently
    computing cumulative E over a vector. A cyclist's Eddington number
    <https://en.wikipedia.org/wiki/Arthur_Eddington#Eddington_number_for_cycling>
    is the maximum number satisfying the condition such that a cyclist has
    ridden E miles or greater on E distinct days. The algorithm in this package
    is an improvement over the conventional approach because both summary
    statistics and cumulative statistics can be computed in linear time, since
    it does not require initial sorting of the data. These functions may also be
    used for computing h-indices for authors, a metric described by Hirsch (2005)
    <doi:10.1073/pnas.0507655102>. Both are specific applications of computing
    the side length of a Durfee square <https://en.wikipedia.org/wiki/Durfee_square>.
    Some additional author-level metrics such as g-index and i10-index are also
    included in the package.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.2.0)
LinkingTo: Rcpp
Imports: Rcpp, R6, methods, xml2
Suggests: testthat, knitr, rmarkdown, stats, dplyr, tibble
SystemRequirements: C++17
VignetteBuilder: knitr
RoxygenNote: 7.3.2
URL: https://github.com/pegeler/eddington2
BugReports: https://github.com/pegeler/eddington2/issues
NeedsCompilation: yes
Packaged: 2026-04-12 03:16:10 UTC; pablo
Author: Paul Egeler [aut, cre],
  Tashi Reigle [ctb]
Maintainer: Paul Egeler <paulegeler@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 21:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 17:06:53 UTC; unix
Archs: eddington.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,E_next)
export(E_cum)
export(E_next)
export(E_num)
export(E_req)
export(E_sat)
export(Eddington)
export(EddingtonModule)
export(durfee)
export(g_index)
export(get_haversine_distance)
export(h_index)
export(i10_index)
export(index)
export(read_gpx)
importFrom(R6,R6Class)
importFrom(Rcpp,loadModule)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
useDynLib(eddington, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# eddington

<!-- badges: start -->

[![CRAN_version](http://www.r-pkg.org/badges/version/eddington)](https://cran.r-project.org/package=eddington)
[![Number_of_Downloads](https://cranlogs.r-pkg.org/badges/grand-total/eddington)](https://cran.r-project.org/package=eddington)
<!-- badges: end -->

One statistic that cyclists are known to track is their Eddington
number. The Eddington number for cycling, *E*, is the maximum number
where a cyclist has ridden *E* miles on *E* distinct days. So to get a
number of 30, you need to have ridden 30 miles or more on 30 separate
days.

This package provides functions to compute a cyclist’s Eddington number,
including efficiently computing cumulative *E* over a vector. These
functions may also be used for *h*-indices for authors. Both are
specific applications of computing the side length of a Durfee square.

The package can also be used to ingest GPS Exchange Format (GPX) files
into a `data.frame` format conducive to computing the Eddington number.

## Installation

You can install the released version of `eddington` from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("eddington")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("remotes")
remotes::install_github("pegeler/eddington2/R/package")
```

## Example

Here is a basic example showing how to get the summary Eddington number
of the included **simulated** `rides` dataset. Note that we first have
to aggregate ride mileage by date.

``` r
library(eddington)
library(dplyr)

rides %>%
  group_by(ride_date) %>%
  summarize(n = n(), total = sum(ride_length)) %>% 
  summarize(E = E_num(total)) %>% 
  pull
#> [1] 29
```

See the package vignette for detailed usage.

NAMESPACES <- c(d = "http://www.topografix.com/GPX/1/1")
TIMESTAMP_FORMAT <- "%FT%T.000Z"

#' Compute the distance between two points using the Haversine formula
#'
#' Uses the Haversine great-circle distance formula to compute the distance
#' between two latitude/longitude points.
#'
#' @param lat_1,lon_1,lat_2,lon_2 The coordinates
#'    used to compute the distance.
#' @param units The units of the output distance.
#' @examples
#' # In NYC, 20 blocks == 1 mile. Thus, computing the distance between two
#' # points along 7th Ave from W 39 St to W 59 St should return ~1 mile.
#' w39_coords <- list(lat=40.75406905512651, lon=-73.98830604245481)
#' w59_coords <- list(lat=40.76684156255418, lon=-73.97908243833855)
#'
#' get_haversine_distance(
#'   w39_coords$lat,
#'   w39_coords$lon,
#'   w59_coords$lat,
#'   w59_coords$lon,
#'   "miles"
#' )
#'
#' # The total distance along a sequence of points can be computed. Consider the
#' # following sequence of points along Park Ave in the form of a list of points
#' # where each point is a list containing a `lat` and `lon` tag.
#' park_ave_coords <- list(
#'   list(lat=40.735337983655434, lon=-73.98973648773142),  # E 15 St
#'   list(lat=40.74772623378332, lon=-73.98066078090876),   # E 35 St
#'   list(lat=40.76026319186414, lon=-73.97149360922498),   # E 55 St
#'   list(lat=40.77301604875587, lon=-73.96217737679450)    # E 75 St
#' )
#'
#' # We can create a function to compute the total distance as follows:
#' compute_total_distance <- function(coords) {
#'   sum(
#'     sapply(
#'       seq_along(coords)[-1],
#'       \(i) get_haversine_distance(
#'         coords[[i]]$lat,
#'         coords[[i]]$lon,
#'         coords[[i - 1]]$lat,
#'         coords[[i - 1]]$lon,
#'         "miles"
#'       )
#'     )
#'   )
#' }
#'
#' # Then applying the function to our sequence results in a total distance.
#' compute_total_distance(park_ave_coords)
#' @returns The distance between two points in the requested units.
#' @references <https://en.wikipedia.org/wiki/Haversine_formula>
#' @export
get_haversine_distance <- function(lat_1,
                                   lon_1,
                                   lat_2,
                                   lon_2,
                                   units = c("miles", "kilometers")) {
  r <- switch(match.arg(units), miles = R_E_MI, kilometers = R_E_KM)
  .Call(`_eddington_get_haversine_distance_`, lat_1, lon_1, lat_2, lon_2, r)
}

#' Read a GPX file into a data frame containing dates and distances
#'
#' Reads in a GPS Exchange Format XML document and outputs a `data.frame`
#' containing distances. The corresponding dates for each track segment
#' (`trkseg`) will be included if present in the source file, else the `date`
#' column will be populated with `NA`s.
#'
#' Distances are computed using the Haversine formula and do not account for
#' elevation changes.
#'
#' This function treats the first timestamp of each `trkseg` as the date of
#' record. Thus overnight track segments will all count toward the day in which
#' the journey began.
#'
#' @param file The input file to be parsed.
#' @param units The units desired for the distance metric.
#' @examples
#' \dontrun{
#' # Get a list of all GPX export files in a directory tree
#' gpx_export_files <- list.files(
#'   "/path/to/gpx/exports/",
#'   pattern = "\\.gpx$",
#'   full.names = TRUE,
#'   recursive = TRUE
#' )
#'
#' # Read in all files and combine them into a single data frame
#' rides <- do.call(rbind, lapply(gpx_export_files, read_gpx))
#' }
#' @returns A data frame containing up to two columns:
#'  \describe{
#'    \item{date}{The date of the ride. See description and details.}
#'    \item{distance}{The distance of the track segment in the requested units.}
#'  }
#' @export
read_gpx <- function(file, units = c("miles", "kilometers")) {
  x <- xml2::read_xml(file)

  distances <- get_trkseg_dist(x, match.arg(units))

  if (!length(distances)) {
    # No trkseg nodes were found. Returning an empty data frame
    return(
      data.frame(
        date = structure(double(0L), class = "Date"),
        distance = double(0L)
      )
    )
  }

  dates <- get_dates(x)

  data.frame(
    date = if (length(dates) == length(distances)) dates else as.Date(NA),
    distance = unlist(distances)
  )
}

# XML parsing helper functions --------------------------------------------

get_dates <- function(x) {
  x |>
    xml2::xml_find_all("//d:trkseg/d:trkpt[1]/d:time", ns = NAMESPACES) |>
    xml2::xml_text() |>
    as.Date.character(TIMESTAMP_FORMAT)
}

get_trkseg_dist <- function(x, units = c("miles", "kilometers")) {
  r <- switch(match.arg(units), miles = R_E_MI, kilometers = R_E_KM)

  lapply(
    xml2::xml_find_all(x, "//d:trkseg", ns = NAMESPACES),
    \(trkseg) {  # get sum of distances for a single trkseg
      trkpts <- xml2::xml_children(trkseg)

      if (length(trkpts) < 2L)
        return(0)

      sum(
        vapply(
          seq_along(trkpts[-1L]),
          \(i) .Call(
            `_eddington_get_haversine_distance_`,
            as.double(xml2::xml_attr(trkpts[[i]], "lat")),
            as.double(xml2::xml_attr(trkpts[[i]], "lon")),
            as.double(xml2::xml_attr(trkpts[[i + 1L]], "lat")),
            as.double(xml2::xml_attr(trkpts[[i + 1L]], "lon")),
            r
          ),
          double(1)
        )
      )
    }
  )
}

# Setting up reference values ---------------------------------------------

# In NYC, 20 blocks == 1 mile and the grid is diagonal. That makes it the
# perfect location for testing the haversine distance formula.
w59_coords <- list(lat=40.76684156255418, lon=-73.97908243833855)  # W 59th St / 7th Ave
w39_coords <- list(lat=40.75406905512651, lon=-73.98830604245481)  # W 39th St / 7th Ave

ave_7_distance <- get_haversine_distance(
  w39_coords$lat, w39_coords$lon, w59_coords$lat, w59_coords$lon, "miles"
)

park_ave_coords <- list(
  list(lat=40.735337983655434, lon=-73.98973648773142),  # E 15 St
  list(lat=40.74772623378332,  lon=-73.98066078090876),  # E 35 St
  list(lat=40.76026319186414,  lon=-73.97149360922498),  # E 55 St
  list(lat=40.77301604875587,  lon=-73.96217737679450)   # E 75 St
)

park_ave_distance <- sum(
  sapply(
    seq_along(park_ave_coords)[-1],
    \(i) get_haversine_distance(
      park_ave_coords[[i]]$lat,
      park_ave_coords[[i]]$lon,
      park_ave_coords[[i - 1]]$lat,
      park_ave_coords[[i - 1]]$lon,
      "miles"
    )
  )
)

context("Haversine distance formula")

test_that("one mile is one mile", {
  d <- get_haversine_distance(
    w39_coords$lat,
    w39_coords$lon,
    w59_coords$lat,
    w59_coords$lon,
    "miles"
  )
  expect_equal(d, 1., tolerance = 0.01)

  d <- get_haversine_distance(
    w39_coords$lat,
    w39_coords$lon,
    w59_coords$lat,
    w59_coords$lon,
    "kilometers"
  )
  expect_equal(d, 1.62, tolerance = 0.01)

})

test_that("d is real for edge cases", {
  # d is only real for 0 <= h <= 1
  # Equal points: h = 1
  expect_equal(eddington:::get_haversine_distance_(0, 0, 0, 0, 1), 0)
  # Antipodal points: h = 1
  expect_equal(eddington:::get_haversine_distance_(-90, -90, 90, 90, 1), pi)
})


context("GPX file processing")

test_that("GPX file base case works", {
  d <- read_gpx("base-case.gpx")
  expect_equal(d$date, as.Date("2023-12-01"))
  expect_equal(d$distance, park_ave_distance)
})

test_that("Multiple trkseg nodes in same file are processed correctly", {
  d <- read_gpx("multi-trkseg.gpx")
  expect_equal(d$date, c(as.Date("2023-12-01"), rep(as.Date("2023-12-02"), 2)))
  expect_equal(sum(d$distance), sum(ave_7_distance, park_ave_distance))
})

test_that("trkseg nodes with 0 or 1 trkpt nodes result in distance of 0", {
  d <- read_gpx("zero-node-trkseg.gpx")
  expect_true(is.na(d$date))
  expect_equal(d$distance, 0.)

  d <- read_gpx("one-node-trkseg.gpx")
  expect_equal(d$date, as.Date("2023-12-01"))
  expect_equal(d$distance, 0.)
})

test_that("Files with missing trkseg nodes return empty data frame", {
  d <- read_gpx("no-trkseg.gpx")
  expect_equal(nrow(d), 0L)
  expect_equal(names(d), c("date", "distance"))
})

test_that("correctly handles time nodes", {
  d <- read_gpx("no-time-nodes.gpx")
  expect_true(is.na(d$date))
  expect_equal(d$distance, park_ave_distance)
})

#' An Rcpp Module for Tracking Eddington Numbers for Cycling
#'
#' A stateful C++ object for computing Eddington numbers.
#'
#' @section Warning:
#'
#' `EddingtonModule` objects cannot be serialized at this time; they cannot be
#' carried between sessions using [base::saveRDS] or [base::save] and then
#' loaded later using [base::readRDS] or [base::load].
#'
#' @param rides An optional vector of values used to initialize the class.
#' @param store_cumulative Whether to store a vector of the cumulative Eddington
#'    number, as accessed from the `cumulative` property.
#' @field new Constructor. Parameter list may either be empty, `store_cumulative`,
#'    or `rides` and `store_cumulative`
#' @field current The current Eddington number.
#' @field cumulative A vector of Eddington numbers or `NULL` if `store_cumulative`
#'    is `FALSE`.
#' @field hashmap A `data.frame` containing the distances and counts above the
#'    current Eddington number.
#' @field update Update the class state with new data.
#' @field getNumberToNext Get the number of additional distances required to
#'    reach the next Eddington number.
#' @field getNumberToTarget Get the number of additional distances required to
#'    reach a target Eddington number.
#'
#' @examples
#' # Create a class instance with some initial data
#' e <- EddingtonModule$new(c(3, 3, 2), store_cumulative = TRUE)
#' e$current
#'
#' # Update with new data and look at the vector of cumulative Eddington numbers.
#' e$update(c(3, 3, 5))
#' e$cumulative
#'
#' # Get the number of rides required to reach the next Eddington number and
#' # an Eddington number of 4.
#' e$getNumberToNext()
#' e$getNumberToTarget(4)
#'
#' @name EddingtonModule
#' @importFrom methods new
#' @importFrom Rcpp loadModule
#' @export EddingtonModule
Rcpp::loadModule("eddington_mod", TRUE)

filter_na <- \(x) Filter(Negate(is.na), x)

#' Define a custom bibliometric index function
#'
#' @param f A function to be applied to the index before comparison.
#' @param cumulative A logical on whether to apply a cumulative sum to the counts.
#' @examples
#' # NOTE: These will all be less performant than their counterparts exported
#' # in this package, i.e., `h_index()`, `g_index()`, `i10_index()`.
#' set.seed(2018)
#' citations <- rgamma(30, shape = 2, scale = 10)
#'
#' # Create an h-index
#' my_h_index <- index(force)
#' my_h_index(citations)
#'
#' # Create a g-index function
#' my_g_index <- index(\(i) i * i, cumulative = TRUE)
#' my_g_index(citations)
#'
#' # Create an i10-index
#' my_i10_index <- index(\(i) 10L)
#' my_i10_index(citations)
#'
#' @returns A function that will compute the specified index.
#' @seealso [bibliometrics]
#' @export
index <- function(f, cumulative = FALSE) {
  force(f)
  force(cumulative)
  function(xs) {
    is <- vapply(seq_along(xs), f, integer(1L))
    xs <- sort(xs, decreasing = TRUE)
    sum((if (cumulative) cumsum(xs) else xs) >= is)
  }
}

#' Compute several bibliometric indices
#'
#' Compute bibliometric indices such as the h-index, g-index, and i10-index.
#'
#' @section Implicit Type Conversions:
#' The `h_index()` function implicitly coerces inputs into integer vectors,
#' which will truncate any floating point inputs. This usually will result in
#' expected outputs, as there are not typically fractional inputs in the
#' intended domain, and the definitions of these indices are defined on integral
#' thresholds explicitly. However, to maximize the versatility of g-index
#' computation, the `g_index()` function does not perform this integer coercion.
#' Therefore it is worth noting that floating point input can push the g-index
#' higher on edge cases. For example,
#' `g_index(as.integer(daily_totals$total_length)) !=
#' g_index(daily_totals$total_length)` Thus to ensure accurate g-index results
#' on data that may have a fractional component, it is advised to first perform
#' an integer conversion prior to passing a vector into `g_index()` or otherwise
#' validate inputs.
#'
#' This integer conversion will also cause the `h_index()` to fail when inputs
#' contain extremely large values (\eqn{> 2^{31} - 1}{> 2^31 - 1}). The
#' Eddington number family of functions and `durfee()` do not have this check,
#' and may result in inaccurate outputs.
#'
#' @param citations A vector of citation counts.
#' @param na.rm If `TRUE`, `NA` values will be filtered out. Otherwise, any `NA`
#'   value found in the vector will propagate and `NA` will be returned.
#' @param is_sorted Whether the data is pre-sorted in descending order. This may
#'   speed up computations for some algorithms. The pre-sorted assumption is
#'   tested and a warning is emitted if unsorted data is detected.
#' @returns The summary number.
#' @references <https://en.wikipedia.org/wiki/Author-level_metrics>,
#' <https://en.wikipedia.org/wiki/G-index>
#' @seealso [E_num()], [durfee()]
#' @name h_index
#' @aliases bibliometrics
#' @export
h_index <- function(citations, na.rm = FALSE) {
  has_na <- anyNA(citations)
  if (!na.rm && has_na)
    return(NA_integer_)

  if (is.double(citations) && any(citations > .Machine$integer.max, na.rm = TRUE))
    stop("Values exceed limits for the integer data type.")

  .Call(`_eddington_E_num`, if (has_na) filter_na(citations) else citations)
}

#' @rdname h_index
#' @export
i10_index <- function(citations, na.rm = FALSE) sum(citations >= 10L, na.rm = na.rm)

#' @rdname h_index
#' @export
g_index <- function(citations, na.rm = FALSE, is_sorted = FALSE) {
  has_na <- anyNA(citations)
  if (!na.rm && has_na)
    return(NA_integer_)

  .Call(
    `_eddington_g_index_`,
    if (has_na) filter_na(citations) else citations,
    is_sorted
  )
}

context("Bibliometrics")

data("daily_totals", package = "eddington")

# Reference: https://en.wikipedia.org/wiki/G-index
cites <- tibble::tribble(
  ~row, ~author1, ~author2,
  1, 30, 10,
  2, 17, 9,
  3, 15, 9,
  4, 13, 9,
  5, 8, 8,
  6, 6, 6,
  7, 5, 5,
  8, 4, 4,
  9, 3, 2,
  10, 1, 1
)

test_that("index factory function works", {
  xs <- daily_totals$total_length

  h_alt <- index(force)
  expect_identical(h_alt(xs), h_index(xs))

  g_alt <- index(\(i) i * i, cumulative = TRUE)
  expect_identical(g_alt(xs), g_index(xs))
  expect_identical(g_alt(as.integer(xs)), g_index(as.integer(xs)))

  i10_alt <- index(\(i) 10L)
  expect_identical(i10_alt(xs), i10_index(xs))
})

test_that("h-index works", {
  expect_identical(h_index(cites$author1), 6L)
  expect_identical(h_index(cites$author2), 6L)

  # NA handling
  expect_identical(h_index(c(1:3, NA), na.rm = TRUE), 2L)
  expect_identical(h_index(c(1:3, NA), na.rm = FALSE), NA_integer_)

  # Large value check
  expect_error(h_index(3**31), "exceed limits")
  expect_error(h_index(c(3**31, NA_real_), na.rm = TRUE), "exceed limits")
  expect_identical(h_index(c(3**31, NA_real_), na.rm = FALSE), NA_integer_)
})

test_that("i10-index works", {
  expect_identical(i10_index(cites$author1), 4L)
  expect_identical(i10_index(cites$author2), 1L)
})


test_that("g-index works", {
  d <- daily_totals$total_length
  expect_identical(g_index(d), 38L)
  expect_identical(g_index(as.integer(d)), 37L)

  expect_identical(g_index(cites$author1, is_sorted = TRUE), 10L)
  expect_identical(g_index(cites$author2, is_sorted = TRUE), 7L)

  # Edge cases
  expect_identical(g_index(integer()), 0L)

  # NA handling
  expect_identical(g_index(c(1:3, NA), na.rm = TRUE), 2L)
  expect_identical(g_index(c(1:3, NA), na.rm = FALSE), NA_integer_)

  # Sort check
  expect_no_warning(g_index(1:3))  # sort is default
  # If caller claims data is pre-sorted, we still check and emit warning if
  # unsorted data is observed.
  expect_warning(
    g_index(1:3, is_sorted = TRUE),
    "unsorted"
  )
})

#' @useDynLib eddington, .registration = TRUE
#' @importFrom Rcpp sourceCpp
NULL

# Function definitions ------------------------------------------------------

#' Determine if a dataset satisfies a specified Eddington number
#'
#' Indicates whether a certain Eddington number is satisfied, given the data.
#'
#' @inheritParams E_num
#' @param candidate The Eddington number to test for.
#' @seealso \code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_num}},
#'   \code{\link{E_req}}
#' @return A logical vector of length 1.
#' @export
E_sat <- function(rides, candidate) {
  length(rides) >= candidate && sum(rides >= candidate) >= candidate
}

#' Determine the number of additional rides required to achieve a specified
#' Eddington number
#'
#' Determine the number of additional rides required to achieve a specified
#' Eddington number.
#'
#' @inheritParams E_num
#' @param candidate The Eddington number to test for.
#' @seealso \code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_num}},
#'   \code{\link{E_sat}}
#' @return An integer vector of length 1. Returns \code{0L} if \emph{E} is
#'   already achieved.
#' @export
E_req <- function(rides, candidate) {
  max(as.integer(candidate) - sum(rides >= candidate), 0L)
}

# Custom print methods -------------------------------------------------------
#' @export
print.E_next <- function(x, ...) {

  out_string <- sprintf(
    "Your current Eddington Number is %i. You need %i %s of %i or greater to get to an Eddington number of %i.",
    x$E,
    x$req,
    if (x$req > 1) "rides" else "ride",
    x$E + 1,
    x$E + 1
  )

  cat(strwrap(out_string), sep = "\n")

}

#' An R6 Class for Tracking Eddington Numbers for Cycling
#'
#' @description
#' The class will maintain the state of the algorithm, allowing for efficient
#' updates as new rides come in.
#'
#' @section Warnings:
#'
#' The implementation uses an experimental base R feature [utils::hashtab].
#'
#' Cloning of `Eddington` objects is disabled. Additionally, `Eddington` objects
#' cannot be serialized; they cannot be carried between sessions using
#' [base::saveRDS] or [base::save] and then loaded later using [base::readRDS]
#' or [base::load].
#'
#' @examples
#' # Randomly generate a set of 15 rides
#' rides <- rgamma(15, shape = 2, scale = 10)
#'
#' # View the rides sorted in decreasing order
#' stats::setNames(sort(rides, decreasing = TRUE), seq_along(rides))
#'
#' # Create the Eddington object
#' e <- Eddington$new(rides, store.cumulative = TRUE)
#'
#' # Get the Eddington number
#' e$current
#'
#' # Update with new data
#' e$update(rep(25, 10))
#'
#' # See the new data
#' e$cumulative
#' @importFrom R6 R6Class
#' @export
Eddington <- R6::R6Class(
  "Eddington",

  # Not cloneable because it will not make a deep copy of the `hashtab` object.
  cloneable = FALSE,

  public = list(

    #' @description
    #' Create a new Eddington object.
    #' @param rides A vector of rides
    #' @param store.cumulative logical, indicating whether to keep a vector of
    #'   cumulative Eddington numbers
    #' @return A new `Eddington` object
    initialize = function(rides, store.cumulative = FALSE) {
      private$.hashmap <- hashtab(size = 150L)
      if (store.cumulative)
        private$.cumulative <- integer(0L)
      if (!missing(rides))
        self$update(rides)
    },

    #' @description
    #' Print the current Eddington number.
    print = function() {
      cat("Current Eddington Number is:", private$.running, "\n")
      invisible(self)
    },

    #' @description
    #' Add new rides to the existing `Eddington` object.
    #' @param rides A vector of rides
    update = function(rides) {
        private$.n <- private$.n + length(rides)
        rides <- as.integer(rides)
        if (!is.null(private$.cumulative)) {
          cumulative <- integer(length(rides))
        }

        for (i in seq_along(rides)) {
          ride <- rides[i]

          if (ride > private$.running) {
            private$.above <- private$.above + 1L
            private$.incrementHashmap(ride)

            if (private$.above > private$.running) {
              private$.running <- private$.running + 1L
              n_above_at_dist <- gethash(private$.hashmap, private$.running, 0L)
              private$.above <- private$.above - n_above_at_dist
              remhash(private$.hashmap, private$.running);
            }
        }

        if (!is.null(private$.cumulative)) {
          cumulative[i] <- private$.running
        }
      }
      if (!is.null(private$.cumulative)) {
        private$.cumulative <- c(private$.cumulative, cumulative)
      }
      invisible(self)
    },

    #' @description
    #' Get the number of rides of a specified length to get to a target
    #' Eddington number.
    #' @param target Target Eddington number
    #' @return An integer representing the number of rides of target length
    #'   needed to achieve the target number.
    getNumberToTarget = function(target) {
      if (target <= private$.running)
        return(0L)
      if (target == private$.running + 1L)
        return(self$number_to_next)
      n_above <- 0L
      maphash(
        private$.hashmap,
        \(k, v) if (k >= target) n_above <<- n_above + v
      )
      target - n_above
    },

    #' @description
    #' Test if an Eddington number is satisfied.
    #' @param target Target Eddington number
    #' @return Logical
    isSatisfied = function(target) private$.running >= target
  ),

  active = list(
    #' @field current The current Eddington number.
    current = function(value) {
      if (!missing(value))
        stop("Data member is read only.", call. = FALSE)
      private$.running
    },

    #' @field cumulative A vector of cumulative Eddington numbers.
    cumulative = function(value) {
      if (is.null(private$.cumulative))
        stop("Cumulative Eddington Number is not being tracked", call. = FALSE)
      if (!missing(value))
        stop("Data member is read only.", call. = FALSE)
      private$.cumulative
    },

    #' @field number_to_next The number of rides needed to get to the next
    #'  Eddington number.
    number_to_next = function(value) {
      if (!missing(value))
        stop("Data member is read only.", call. = FALSE)
      private$.running + 1L - private$.above
    },

    #' @field n The number of rides in the data.
    n = function(value) {
      if (!missing(value))
        stop("Data member is read only.", call. = FALSE)
      private$.n
    },

    #' @field hashmap The hash map of rides above the current Eddington number.
    hashmap = function(value) {
      if (!missing(value))
        stop("Data member is read only.", call. = FALSE)
      nkeys <- numhash(private$.hashmap)
      i <- 1L
      lengths <- integer(nkeys)
      counts <- integer(nkeys)
      maphash(
        private$.hashmap,
        \(k, v) {
          lengths[i] <<- k
          counts[i] <<- v
          i <<- i + 1L
        }
      )
      h <- data.frame("length" = lengths, "count" = counts)
      h[order(h$length),]
    }
  ),

  private = list(
    .running = 0L,
    .above = 0L,
    .n = 0L,
    .cumulative = NULL,
    .hashmap = NULL,

    .incrementHashmap = function(ride) {
      old <- gethash(private$.hashmap, ride, 0L)
      sethash(private$.hashmap, ride, old + 1L)
    }
  )
)

context("Package functions")

simdata <- readRDS("simdata.rds")
refdata <- readRDS("refdata.rds")

E_ref <- refdata[length(refdata)]

test_that("E_num works",{
  expect_identical(E_num(simdata), E_ref)
})

test_that("E_cum works",{
  expect_identical(E_cum(simdata), refdata)
})


test_that("E_sat works",{
  expect_true(E_sat(simdata, E_ref))
  expect_false(E_sat(simdata, E_ref + 1L))
})

test_that("E_next works",{
  x <- E_next(simdata)
  ref_next <- x$E + 1L
  enough_rides <- c(simdata, rep(ref_next, x$req))
  not_enough_rides <- c(simdata, rep(ref_next, x$req - 1))

  expect_true(E_sat(enough_rides, ref_next))
  expect_false(E_sat(not_enough_rides, ref_next))
})

test_that("E_req works",{
  x <- E_next(simdata)
  ref_next <- x$E + 1L
  expect_equal(E_req(simdata, ref_next), x$req)
})

context("R6 class")

test_that("Eddington class works",{

  # constructor
  e <- Eddington$new(store.cumulative = FALSE)
  e$update(1:3)
  expect_equal(e$current, 2L)
  expect_error(e$cumulative)

  e <- Eddington$new(simdata, store.cumulative = TRUE)

  # current
  expect_equal(e$current, E_ref)

  # cumulative
  expect_equal(e$cumulative, E_cum(simdata))

  # Number to next and target
  expect_equal(e$number_to_next, E_next(simdata)$req)
  expect_equal(e$getNumberToTarget(27), E_req(simdata, 27))

  # satisfied
  expect_true(e$isSatisfied(e$current))
  expect_true(e$isSatisfied(e$current - 1L))
  expect_false(e$isSatisfied(e$current + 1L))

  # n
  expect_equal(e$n, length(simdata))

  # Get hashmap
  hashmap <- e$hashmap
  expect_s3_class(e$hashmap, "data.frame")
  expect_true(all(hashmap[["length"]] > e$current))
  expect_true(all(names(hashmap) == c("length", "count")))
  expect_true(nrow(hashmap) > 0)

  # clone is disabled
  expect_false(exists("clone", envir = e))

  # update
  e <- Eddington$new(simdata, store.cumulative = TRUE)
  e$update(rep(25, 10))
  expect_equal(e$cumulative, E_cum(c(simdata, rep(25, 10))))

})


context("Rcpp Module class")

test_that("EddingtonModule class works",{

  # constuctors
  e <- EddingtonModule$new()

  e <- EddingtonModule$new(TRUE)
  e$update(1:3)
  expect_true(!is.null(e$cumulative))

  e <- EddingtonModule$new(FALSE)
  e$update(1:3)
  expect_true(is.null(e$cumulative))

  e <- EddingtonModule$new(simdata, store_cumulative = TRUE)

  # current
  expect_equal(e$current, E_ref)

  # cumulative
  expect_equal(e$cumulative, E_cum(simdata))

  # Number to next and target
  expect_equal(e$getNumberToNext(), E_next(simdata)$req)
  expect_equal(e$getNumberToTarget(27), E_req(simdata, 27))

  # Get hashmap
  hashmap <- e$hashmap
  expect_s3_class(e$hashmap, "data.frame")
  expect_true(all(hashmap[["length"]] > e$current))
  expect_true(all(names(hashmap) == c("length", "count")))
  expect_true(nrow(hashmap) > 0)

  # update
  e <- EddingtonModule$new(simdata, TRUE)
  e$update(rep(25, 10))
  expect_equal(e$cumulative, E_cum(c(simdata, rep(25, 10))))

})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

g_index_ <- function(citations, is_sorted = FALSE) {
    .Call(`_eddington_g_index_`, citations, is_sorted)
}

#' Get the Eddington number for cycling
#'
#' Gets the \href{https://en.wikipedia.org/wiki/Arthur_Eddington#Eddington_number_for_cycling}{Eddington number for cycling}.
#' The Eddington Number for cycling, \emph{E}, is the maximum number where a
#' cyclist has ridden \emph{E} miles on \emph{E} distinct days.
#'
#' The Eddington Number for cycling is related to computing the rank of an
#' integer partition, which is the same as computing the side length of its
#' \href{https://en.wikipedia.org/wiki/Durfee_square}{Durfee square}. Another
#' relevant application of this metric is computing the Hirsch index
#' (\doi{10.1073/pnas.0507655102}) for publications.
#'
#' This is not to be confused with the
#' \href{https://en.wikipedia.org/wiki/Eddington_number}{Eddington Number in
#' astrophysics}, \eqn{N_{Edd}}, which represents the number of protons in the
#' observable universe.
#'
#' @param rides A vector of mileage, where each element represents a single
#'   day.
#'
#' @return An integer which is the Eddington cycling number for the
#'   data provided.
#'
#' @seealso \code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_req}},
#'   \code{\link{E_sat}}
#'
#' @examples
#' # Randomly generate a set of 15 rides
#' rides <- rgamma(15, shape = 2, scale = 10)
#'
#' # View the rides sorted in decreasing order
#' stats::setNames(sort(rides, decreasing = TRUE), seq_along(rides))
#'
#' # Get the Eddington number
#' E_num(rides)
#' @export
E_num <- function(rides) {
    .Call(`_eddington_E_num`, rides)
}

#' Calculate the cumulative Eddington number
#'
#' This function is much like \code{\link{E_num}} except it provides
#' a cumulative Eddington number over the vector rather than a single summary
#' number.
#'
#' @inheritParams E_num
#' @seealso \code{\link{E_next}}, \code{\link{E_num}}, \code{\link{E_req}},
#'   \code{\link{E_sat}}
#' @return An integer vector the same length as \code{rides}.
#' @export
E_cum <- function(rides) {
    .Call(`_eddington_E_cum`, rides)
}

#' Get the number of rides required to increment to the next Eddington number
#'
#' Get the number of rides required to increment to the next Eddington number.
#'
#' @inheritParams E_num
#' @seealso \code{\link{E_cum}}, \code{\link{E_num}}, \code{\link{E_req}},
#'   \code{\link{E_sat}}
#' @return A named list with the current Eddington number (\code{E}) and the
#'   number of rides required to increment by one (\code{req}).
#' @export
E_next <- function(rides) {
    .Call(`_eddington_E_next`, rides)
}

get_haversine_distance_ <- function(lat_1, lon_1, lat_2, lon_2, r) {
    .Call(`_eddington_get_haversine_distance_`, lat_1, lon_1, lat_2, lon_2, r)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_eddington_RcppExport_registerCCallable`)
})

#' Compute the side length of a Durfee square
#'
#' @param is An integer vector representing an integer partition.
#' @returns The side length of the Durfee square for that partition.
#' @export
durfee <- function(is) .Call(`_eddington_E_num`, is)

# Using .Call instead of just assigning `E_num` to `durfee` so that formals
# could be renamed.

#' A year of simulated bicycle ride mileages
#'
#' Simulated dates and distances of rides occurring in 2009.
#'
#' The dataset contains a total of 3,419 miles spread across 178
#' unique days. The Eddington number for the year was 29.
#'
#' @format A data frame with 250 rows and 2 variables:
#' \describe{
#'   \item{ride_date}{date the ride occurred}
#'   \item{ride_length}{the length in miles}
#' }
#' @seealso [daily_totals]
"rides"


#' A year of simulated bicycle ride mileages, aggregated by day
#'
#' Simulated dates and distances of rides occurring in 2009. This is an
#' aggregation of the \code{\link{rides}} dataset by day.
#'
#' The dataset contains a total of 3,419 miles spread across 178
#' unique days. The Eddington number for the year was 29.
#'
#' @format A data frame with 178 rows and 2 variables:
#' \describe{
#'   \item{ride_date}{date the ride occurred}
#'   \item{total_length}{the total length in miles for each day}
#' }
#' @seealso [rides]
"daily_totals"

---
title: "Computing your Eddington number using the `eddington` package."
author:
  "Paul W. Egeler, M.S."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{eddington}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5,
  fig.align = "center"
)

# Putting this here too so that we don't have pkg startup messages.
library(dplyr)
```

## Introduction

One statistic that cyclists might be interested to track is their Eddington
number. The Eddington number for cycling, _E_, is the maximum number where a
cyclist has ridden _E_ miles on _E_ distinct days. So to get a number of 30, you
need to have ridden 30 miles or more on 30 separate days.

This package allows the user to compute Eddington numbers and more. For example,
users can determine if a specific Eddington number is satisfied or how many
rides of the appropriate distance are needed to increment their Eddington
number. It also contains **simulated** data to demonstrate the package use.

## The Data

Loading the simulated data is simple. Let's take a quick look at the first few lines.

```{r setup}
library(eddington)
head(rides)
```

First, we need to establish the granularity of the data. As you can see above,
there are at least two entries for 2009-01-18. Since this data simulates a rider
who tracked each individual ride, there could be more than one ride per day in
this dataset. Therefore, we need to transform the data to aggregate on day.

```{r xform}
library(dplyr)

days <- rides %>%
  group_by(ride_date) %>%
  summarize(n = n(), total = sum(ride_length))

head(days)
```

Let's just take a quick peek at the summary stats:

```{r summary}
summary(days)
```


### Histogram of Daily Mileages

This plot provides a histogram of daily mileages. Note the summary Eddington
number is in dark red---we'll see how that's calculated in the next section.

```{r, echo=FALSE}
hist(
  as.integer(days$total), 
  breaks = 30, 
  main = "Histogram of Daily Mileages", 
  xlab = "Miles"
)

abline(v = E_num(days$total), col = "darkred")

legend(
  "topright",
  legend = "Eddington Number",
  col = "darkred",
  bty = "n",
  lty = 1L
)
```

## Computing Eddington Numbers

To compute the Eddington number, we use the `E_num()` function like so:

```{r enum}
E_num(days$total)
```

### Cumulative E

To see how the Eddington number progressed over the year, use `E_cum()`. It can
be useful to add the vector as a new column onto the existing dataset:

```{r ecum}
days$E <- E_cum(days$total)

head(days)
```

It might be more interesting to see that graphically:

```{r needle, echo=FALSE}
E <- E_num(days$total)

E_contribs <- days[days$total >= E,]


plot(
  y = days$total,
  x = days$ride_date,
  type = "h",
  main = "Ride Mileages in 2009",
  xlab = "Ride Day",
  ylab = "Total Miles",
  bty = "n",
  ylim = c(0, 90)
)


lines(
  y = c(0, days$E),
  x = c(as.Date("2009-01-01"), days$ride_date),
  type = "s",
  col = "darkred"
)

abline(h = E, lty = 2L, col = "darkred")

text(
  E_contribs[,c("ride_date","total")],
  labels = as.integer(E_contribs[["total"]]),
  pos = 3,
  cex = 0.7
  )

legend(
  "topleft",
  title = "Eddington Number",
  legend = c("Cumulative", "Summary"),
  col = "darkred",
  bty = "n",
  lty = c(1L, 2L)
)
```

## Addtional Functionality

### Incrementing to the Next Eddington Number

So now that we know that the summary Eddington number was 29 for the year, let's
see how many more rides of length 30 or greater that we would have needed to
increment the _E_ to 30.

```{r enext}
E_next(days$total)
```

### Stretch Goals

An ambitious rider might be interested to see the number of rides required to
reach a stretch goal. Say, how many more rides would have been needed to reach
an _E_ of 50? For that, we use `E_req()`, which stands for "required."

```{r ereq}
E_req(days$total, 50)
```

### Check if a Dataset Satisfies an Arbitrary _E_

We could also check to see if we've gotten to 30 by using `E_sat()`, which
stands for "satisfies."

```{r esat}
E_sat(days$total, 30)
```

## Conclusion

The text above should give you a good start in using the `eddington` package.
Although this package was developed with bicycling in mind, it has applications
for other users as well. The Eddington number is a specific application of
computing the side length of a Durfee square. Another application is the Hirsch
index, or _h_-index, which a popular number in bibliometrics.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/durfee.R
\name{durfee}
\alias{durfee}
\title{Compute the side length of a Durfee square}
\usage{
durfee(is)
}
\arguments{
\item{is}{An integer vector representing an integer partition.}
}
\value{
The side length of the Durfee square for that partition.
}
\description{
Compute the side length of a Durfee square
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{E_cum}
\alias{E_cum}
\title{Calculate the cumulative Eddington number}
\usage{
E_cum(rides)
}
\arguments{
\item{rides}{A vector of mileage, where each element represents a single
day.}
}
\value{
An integer vector the same length as \code{rides}.
}
\description{
This function is much like \code{\link{E_num}} except it provides
a cumulative Eddington number over the vector rather than a single summary
number.
}
\seealso{
\code{\link{E_next}}, \code{\link{E_num}}, \code{\link{E_req}},
\code{\link{E_sat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rides.r
\docType{data}
\name{rides}
\alias{rides}
\title{A year of simulated bicycle ride mileages}
\format{
A data frame with 250 rows and 2 variables:
\describe{
\item{ride_date}{date the ride occurred}
\item{ride_length}{the length in miles}
}
}
\usage{
rides
}
\description{
Simulated dates and distances of rides occurring in 2009.
}
\details{
The dataset contains a total of 3,419 miles spread across 178
unique days. The Eddington number for the year was 29.
}
\seealso{
\link{daily_totals}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibliometrics.R
\name{h_index}
\alias{h_index}
\alias{bibliometrics}
\alias{i10_index}
\alias{g_index}
\title{Compute several bibliometric indices}
\usage{
h_index(citations, na.rm = FALSE)

i10_index(citations, na.rm = FALSE)

g_index(citations, na.rm = FALSE, is_sorted = FALSE)
}
\arguments{
\item{citations}{A vector of citation counts.}

\item{na.rm}{If \code{TRUE}, \code{NA} values will be filtered out. Otherwise, any \code{NA}
value found in the vector will propagate and \code{NA} will be returned.}

\item{is_sorted}{Whether the data is pre-sorted in descending order. This may
speed up computations for some algorithms. The pre-sorted assumption is
tested and a warning is emitted if unsorted data is detected.}
}
\value{
The summary number.
}
\description{
Compute bibliometric indices such as the h-index, g-index, and i10-index.
}
\section{Implicit Type Conversions}{

The \code{h_index()} function implicitly coerces inputs into integer vectors,
which will truncate any floating point inputs. This usually will result in
expected outputs, as there are not typically fractional inputs in the
intended domain, and the definitions of these indices are defined on integral
thresholds explicitly. However, to maximize the versatility of g-index
computation, the \code{g_index()} function does not perform this integer coercion.
Therefore it is worth noting that floating point input can push the g-index
higher on edge cases. For example,
\code{g_index(as.integer(daily_totals$total_length)) != g_index(daily_totals$total_length)} Thus to ensure accurate g-index results
on data that may have a fractional component, it is advised to first perform
an integer conversion prior to passing a vector into \code{g_index()} or otherwise
validate inputs.

This integer conversion will also cause the \code{h_index()} to fail when inputs
contain extremely large values (\eqn{> 2^{31} - 1}{> 2^31 - 1}). The
Eddington number family of functions and \code{durfee()} do not have this check,
and may result in inaccurate outputs.
}

\references{
\url{https://en.wikipedia.org/wiki/Author-level_metrics},
\url{https://en.wikipedia.org/wiki/G-index}
}
\seealso{
\code{\link[=E_num]{E_num()}}, \code{\link[=durfee]{durfee()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eddington-mod.r
\name{EddingtonModule}
\alias{EddingtonModule}
\title{An Rcpp Module for Tracking Eddington Numbers for Cycling}
\arguments{
\item{rides}{An optional vector of values used to initialize the class.}

\item{store_cumulative}{Whether to store a vector of the cumulative Eddington
number, as accessed from the \code{cumulative} property.}
}
\description{
A stateful C++ object for computing Eddington numbers.
}
\section{Fields}{

\describe{
\item{\code{new}}{Constructor. Parameter list may either be empty, \code{store_cumulative},
or \code{rides} and \code{store_cumulative}}

\item{\code{current}}{The current Eddington number.}

\item{\code{cumulative}}{A vector of Eddington numbers or \code{NULL} if \code{store_cumulative}
is \code{FALSE}.}

\item{\code{hashmap}}{A \code{data.frame} containing the distances and counts above the
current Eddington number.}

\item{\code{update}}{Update the class state with new data.}

\item{\code{getNumberToNext}}{Get the number of additional distances required to
reach the next Eddington number.}

\item{\code{getNumberToTarget}}{Get the number of additional distances required to
reach a target Eddington number.}
}}

\section{Warning}{


\code{EddingtonModule} objects cannot be serialized at this time; they cannot be
carried between sessions using \link[base:readRDS]{base::saveRDS} or \link[base:save]{base::save} and then
loaded later using \link[base:readRDS]{base::readRDS} or \link[base:load]{base::load}.
}

\examples{
# Create a class instance with some initial data
e <- EddingtonModule$new(c(3, 3, 2), store_cumulative = TRUE)
e$current

# Update with new data and look at the vector of cumulative Eddington numbers.
e$update(c(3, 3, 5))
e$cumulative

# Get the number of rides required to reach the next Eddington number and
# an Eddington number of 4.
e$getNumberToNext()
e$getNumberToTarget(4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpx.r
\name{get_haversine_distance}
\alias{get_haversine_distance}
\title{Compute the distance between two points using the Haversine formula}
\usage{
get_haversine_distance(
  lat_1,
  lon_1,
  lat_2,
  lon_2,
  units = c("miles", "kilometers")
)
}
\arguments{
\item{lat_1, lon_1, lat_2, lon_2}{The coordinates
used to compute the distance.}

\item{units}{The units of the output distance.}
}
\value{
The distance between two points in the requested units.
}
\description{
Uses the Haversine great-circle distance formula to compute the distance
between two latitude/longitude points.
}
\examples{
# In NYC, 20 blocks == 1 mile. Thus, computing the distance between two
# points along 7th Ave from W 39 St to W 59 St should return ~1 mile.
w39_coords <- list(lat=40.75406905512651, lon=-73.98830604245481)
w59_coords <- list(lat=40.76684156255418, lon=-73.97908243833855)

get_haversine_distance(
  w39_coords$lat,
  w39_coords$lon,
  w59_coords$lat,
  w59_coords$lon,
  "miles"
)

# The total distance along a sequence of points can be computed. Consider the
# following sequence of points along Park Ave in the form of a list of points
# where each point is a list containing a `lat` and `lon` tag.
park_ave_coords <- list(
  list(lat=40.735337983655434, lon=-73.98973648773142),  # E 15 St
  list(lat=40.74772623378332, lon=-73.98066078090876),   # E 35 St
  list(lat=40.76026319186414, lon=-73.97149360922498),   # E 55 St
  list(lat=40.77301604875587, lon=-73.96217737679450)    # E 75 St
)

# We can create a function to compute the total distance as follows:
compute_total_distance <- function(coords) {
  sum(
    sapply(
      seq_along(coords)[-1],
      \(i) get_haversine_distance(
        coords[[i]]$lat,
        coords[[i]]$lon,
        coords[[i - 1]]$lat,
        coords[[i - 1]]$lon,
        "miles"
      )
    )
  )
}

# Then applying the function to our sequence results in a total distance.
compute_total_distance(park_ave_coords)
}
\references{
\url{https://en.wikipedia.org/wiki/Haversine_formula}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibliometrics.R
\name{index}
\alias{index}
\title{Define a custom bibliometric index function}
\usage{
index(f, cumulative = FALSE)
}
\arguments{
\item{f}{A function to be applied to the index before comparison.}

\item{cumulative}{A logical on whether to apply a cumulative sum to the counts.}
}
\value{
A function that will compute the specified index.
}
\description{
Define a custom bibliometric index function
}
\examples{
# NOTE: These will all be less performant than their counterparts exported
# in this package, i.e., `h_index()`, `g_index()`, `i10_index()`.
set.seed(2018)
citations <- rgamma(30, shape = 2, scale = 10)

# Create an h-index
my_h_index <- index(force)
my_h_index(citations)

# Create a g-index function
my_g_index <- index(\(i) i * i, cumulative = TRUE)
my_g_index(citations)

# Create an i10-index
my_i10_index <- index(\(i) 10L)
my_i10_index(citations)

}
\seealso{
\link{bibliometrics}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eddington.r
\name{E_sat}
\alias{E_sat}
\title{Determine if a dataset satisfies a specified Eddington number}
\usage{
E_sat(rides, candidate)
}
\arguments{
\item{rides}{A vector of mileage, where each element represents a single
day.}

\item{candidate}{The Eddington number to test for.}
}
\value{
A logical vector of length 1.
}
\description{
Indicates whether a certain Eddington number is satisfied, given the data.
}
\seealso{
\code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_num}},
\code{\link{E_req}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpx.r
\name{read_gpx}
\alias{read_gpx}
\title{Read a GPX file into a data frame containing dates and distances}
\usage{
read_gpx(file, units = c("miles", "kilometers"))
}
\arguments{
\item{file}{The input file to be parsed.}

\item{units}{The units desired for the distance metric.}
}
\value{
A data frame containing up to two columns:
\describe{
\item{date}{The date of the ride. See description and details.}
\item{distance}{The distance of the track segment in the requested units.}
}
}
\description{
Reads in a GPS Exchange Format XML document and outputs a \code{data.frame}
containing distances. The corresponding dates for each track segment
(\code{trkseg}) will be included if present in the source file, else the \code{date}
column will be populated with \code{NA}s.
}
\details{
Distances are computed using the Haversine formula and do not account for
elevation changes.

This function treats the first timestamp of each \code{trkseg} as the date of
record. Thus overnight track segments will all count toward the day in which
the journey began.
}
\examples{
\dontrun{
# Get a list of all GPX export files in a directory tree
gpx_export_files <- list.files(
  "/path/to/gpx/exports/",
  pattern = "\\\\.gpx$",
  full.names = TRUE,
  recursive = TRUE
)

# Read in all files and combine them into a single data frame
rides <- do.call(rbind, lapply(gpx_export_files, read_gpx))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{E_num}
\alias{E_num}
\title{Get the Eddington number for cycling}
\usage{
E_num(rides)
}
\arguments{
\item{rides}{A vector of mileage, where each element represents a single
day.}
}
\value{
An integer which is the Eddington cycling number for the
data provided.
}
\description{
Gets the \href{https://en.wikipedia.org/wiki/Arthur_Eddington#Eddington_number_for_cycling}{Eddington number for cycling}.
The Eddington Number for cycling, \emph{E}, is the maximum number where a
cyclist has ridden \emph{E} miles on \emph{E} distinct days.
}
\details{
The Eddington Number for cycling is related to computing the rank of an
integer partition, which is the same as computing the side length of its
\href{https://en.wikipedia.org/wiki/Durfee_square}{Durfee square}. Another
relevant application of this metric is computing the Hirsch index
(\doi{10.1073/pnas.0507655102}) for publications.

This is not to be confused with the
\href{https://en.wikipedia.org/wiki/Eddington_number}{Eddington Number in
astrophysics}, \eqn{N_{Edd}}, which represents the number of protons in the
observable universe.
}
\examples{
# Randomly generate a set of 15 rides
rides <- rgamma(15, shape = 2, scale = 10)

# View the rides sorted in decreasing order
stats::setNames(sort(rides, decreasing = TRUE), seq_along(rides))

# Get the Eddington number
E_num(rides)
}
\seealso{
\code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_req}},
\code{\link{E_sat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rides.r
\docType{data}
\name{daily_totals}
\alias{daily_totals}
\title{A year of simulated bicycle ride mileages, aggregated by day}
\format{
A data frame with 178 rows and 2 variables:
\describe{
\item{ride_date}{date the ride occurred}
\item{total_length}{the total length in miles for each day}
}
}
\usage{
daily_totals
}
\description{
Simulated dates and distances of rides occurring in 2009. This is an
aggregation of the \code{\link{rides}} dataset by day.
}
\details{
The dataset contains a total of 3,419 miles spread across 178
unique days. The Eddington number for the year was 29.
}
\seealso{
\link{rides}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eddington-R6.r
\name{Eddington}
\alias{Eddington}
\title{An R6 Class for Tracking Eddington Numbers for Cycling}
\description{
The class will maintain the state of the algorithm, allowing for efficient
updates as new rides come in.
}
\section{Warnings}{


The implementation uses an experimental base R feature \link[utils:hashtab]{utils::hashtab}.

Cloning of \code{Eddington} objects is disabled. Additionally, \code{Eddington} objects
cannot be serialized; they cannot be carried between sessions using
\link[base:readRDS]{base::saveRDS} or \link[base:save]{base::save} and then loaded later using \link[base:readRDS]{base::readRDS}
or \link[base:load]{base::load}.
}

\examples{
# Randomly generate a set of 15 rides
rides <- rgamma(15, shape = 2, scale = 10)

# View the rides sorted in decreasing order
stats::setNames(sort(rides, decreasing = TRUE), seq_along(rides))

# Create the Eddington object
e <- Eddington$new(rides, store.cumulative = TRUE)

# Get the Eddington number
e$current

# Update with new data
e$update(rep(25, 10))

# See the new data
e$cumulative
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{current}}{The current Eddington number.}

\item{\code{cumulative}}{A vector of cumulative Eddington numbers.}

\item{\code{number_to_next}}{The number of rides needed to get to the next
Eddington number.}

\item{\code{n}}{The number of rides in the data.}

\item{\code{hashmap}}{The hash map of rides above the current Eddington number.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Eddington-new}{\code{Eddington$new()}}
\item \href{#method-Eddington-print}{\code{Eddington$print()}}
\item \href{#method-Eddington-update}{\code{Eddington$update()}}
\item \href{#method-Eddington-getNumberToTarget}{\code{Eddington$getNumberToTarget()}}
\item \href{#method-Eddington-isSatisfied}{\code{Eddington$isSatisfied()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Eddington-new"></a>}}
\if{latex}{\out{\hypertarget{method-Eddington-new}{}}}
\subsection{Method \code{new()}}{
Create a new Eddington object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Eddington$new(rides, store.cumulative = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rides}}{A vector of rides}

\item{\code{store.cumulative}}{logical, indicating whether to keep a vector of
cumulative Eddington numbers}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new \code{Eddington} object
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Eddington-print"></a>}}
\if{latex}{\out{\hypertarget{method-Eddington-print}{}}}
\subsection{Method \code{print()}}{
Print the current Eddington number.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Eddington$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Eddington-update"></a>}}
\if{latex}{\out{\hypertarget{method-Eddington-update}{}}}
\subsection{Method \code{update()}}{
Add new rides to the existing \code{Eddington} object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Eddington$update(rides)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rides}}{A vector of rides}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Eddington-getNumberToTarget"></a>}}
\if{latex}{\out{\hypertarget{method-Eddington-getNumberToTarget}{}}}
\subsection{Method \code{getNumberToTarget()}}{
Get the number of rides of a specified length to get to a target
Eddington number.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Eddington$getNumberToTarget(target)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target Eddington number}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
An integer representing the number of rides of target length
needed to achieve the target number.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Eddington-isSatisfied"></a>}}
\if{latex}{\out{\hypertarget{method-Eddington-isSatisfied}{}}}
\subsection{Method \code{isSatisfied()}}{
Test if an Eddington number is satisfied.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Eddington$isSatisfied(target)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target Eddington number}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Logical
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{E_next}
\alias{E_next}
\title{Get the number of rides required to increment to the next Eddington number}
\usage{
E_next(rides)
}
\arguments{
\item{rides}{A vector of mileage, where each element represents a single
day.}
}
\value{
A named list with the current Eddington number (\code{E}) and the
number of rides required to increment by one (\code{req}).
}
\description{
Get the number of rides required to increment to the next Eddington number.
}
\seealso{
\code{\link{E_cum}}, \code{\link{E_num}}, \code{\link{E_req}},
\code{\link{E_sat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eddington.r
\name{E_req}
\alias{E_req}
\title{Determine the number of additional rides required to achieve a specified
Eddington number}
\usage{
E_req(rides, candidate)
}
\arguments{
\item{rides}{A vector of mileage, where each element represents a single
day.}

\item{candidate}{The Eddington number to test for.}
}
\value{
An integer vector of length 1. Returns \code{0L} if \emph{E} is
already achieved.
}
\description{
Determine the number of additional rides required to achieve a specified
Eddington number.
}
\seealso{
\code{\link{E_cum}}, \code{\link{E_next}}, \code{\link{E_num}},
\code{\link{E_sat}}
}
